<?php
/**
 * RoundCube Calendar
 *
 * CalDAV backend based on exemplary DAViCal / AWL client.
 *
 * @version 0.2 BETA 2
 * @author Michael Duelli
 * @url http://rc-calendar.lazlo.me
 * @version 0.3
 * @author Michael Rasmussen
 * @url http://www.datanom.net
 * @licence GNU GPL
 * @copyright (c) 2010 Lazlo Westerhof - Netherlands
 */
require_once('backend.php');
require_once('caldav-client.php');

final class CalDAV extends Backend 
{
  private $rcmail = null;
  private $cal = null;
  private $calendar = null;
  private $user = null;
  private $collection = null;
  private $vevent = null;
  private $new_vevent = <<<_EOF
BEGIN:VCALENDAR\r
VERSION:2.0\r
PRODID:-//roundcube.net//RoundCube CalDAV Client//EN\r
BEGIN:VEVENT\r
__ICAL__END:VEVENT\r
END:VCALENDAR
_EOF;
  private $move_vevent = <<<_EOF
BEGIN:VCALENDAR\r
VERSION:2.0\r
PRODID:-//roundcube.net//RoundCube CalDAV Client//EN\r
BEGIN:VEVENT\r
__ICAL__END:VEVENT\r
END:VCALENDAR
_EOF;

  /**
   * @param object rcmail   The RoundCube instance.
   * @param string server   The CalDAV server.
   * @param string user     The user name.
   * @param string pass     The user's password.
   * @param string calendar The user calendar.
   */
  public function __construct(
  		$rcmail, $server, $user, $pass, $collection, $calendar) {
    $this->rcmail = $rcmail;
    $this->calendar = '/' . $calendar;
    $new_user = explode('@', $user);
    if (count($new_user) > 1)
	$user = $new_user[0];
    $this->user = $user;
    $this->collection = (strpos($collection, '/') === 0) ? 
    		$collection : "/$collection";
    $this->cal = new CalDAVClient(
    	"$server/$user" . $this->collection,$user, $pass,
    	$calendar /* is ignored currently */);
    $this->cal->setUserAgent('RoundCube');
  }
  
  public function newEvent($start, $end, $summary, $description, $location, $categories, $allDay) {
    $event = "";
    if (!empty($this->rcmail->user->ID)) {
        $uid = hash('sha256', "$summary|$description|".$this->GMT_to_iCalendar($start));
 		$event .= "DTSTAMP:" . $this->GMT_to_iCalendar(date('Y-m-d H:i:s', time()));
 		if ($allDay) {
			$values = $this->getAllDay($start, $end);
        	$event .= "\r\nDTSTART;VALUE=DATE:" . $values->start;
			$event .= "\r\nDTEND;VALUE=DATE:" . $values->end;
		}
 		else {
       		$event .= "\r\nDTSTART:" . $this->GMT_to_iCalendar($start);
       		//$end = date('Y-m-d H:i:s', 
       		//		strtotime("+2 hour", strtotime($start)));
			$event .= "\r\nDTEND:" . $this->GMT_to_iCalendar($end);
		}
		$event .= "\r\nSUMMARY:$summary";
		$event .= "\r\nDESCRIPTION:".$this->nl2ascii($description);
		$event .= "\r\nLOCATION:$location";
		$event .= "\r\nCATEGORIES:$categories";
		$event .= "\r\nUID:$uid\r\n"; 
		$event = str_replace("__ICAL__", $event, $this->new_vevent);
		$uri = "$uid.ics";
		file_put_contents('/tmp/event', "$uri\n$event\n");
		$etag = $this->cal->DoPUTRequest($uri, $event);
    }
  }

  public function editEvent($id, $summary, $description, $location, $categories) {
	if (!empty($this->rcmail->user->ID)) {
    	if (! $this->vevent)
    		$this->vevent = $this->cal->GetEntryByUid($id);
    	if (empty($this->vevent) || $this->vevent == null) {
    		$this->vevent = null;
    		return;
    	}
  		$event = $this->modifyEvent($id, $summary,
  				$description, $location, $categories);
  		if (empty($event))
  			return;
  		$event["DTSTAMP"] =	$this->GMT_to_iCalendar(date('Y-m-d H:i:s', time()));
		$uri = $event['UID'].".ics";
  		$event = $this->implode_hash($event);
  		$event = str_replace("__ICAL__", $event, $this->new_vevent);
    	$etag = $this->cal->DoPUTRequest(
    			$uri, $event, $this->vevent[0]['etag']);
    	$this->event = null;
    }
  }

  public function moveEvent($id, $start, $end, $allDay) {
	if (!empty($this->rcmail->user->ID)) {
	  	if (! $this->vevent)
	    	$this->vevent = $this->cal->GetEntryByUid($id);
	    if ($allDay)
	    	list($start, $end) = $this->adjustTime($start, $end);
   		$this->setTime($id, $start, $end, $allDay);
   		$event = $this->getEvent($this->vevent[0]['data']);
		$uri = $event['UID'].".ics";
  		$event = $this->implode_hash($event);
   		$new_event = str_replace("__ICAL__", $event, $this->move_vevent);
    	$etag = $this->cal->DoPUTRequest(
    			$uri, $new_event, $this->vevent[0]['etag']);
    	$this->vevent = null;
    }
  }
  
  public function resizeEvent($id, $start, $end) {
  	$summary = "";
  	$description = "";
  	$location = "";
  	$categories = "";

	if (!empty($this->rcmail->user->ID)) {
	    $this->vevent = $this->cal->GetEntryByUid($id);
	    $new_event = $this->getEvent($this->vevent[0]['data']);
	    if (! empty($new_event)) {
	    	foreach ($new_event as $key => $item) {
	    		$property = strstr($key, ";", TRUE);
	    		if ($property === FALSE)
	    			$property = $key;
	    		switch (strtoupper($property)) {
	    			case "SUMMARY": $summary = $item; break;
	    			case "DESCRIPTION": $description = $item; break;
	    			case "LOCATION": $location = $item; break;
	    			case "CATEGORIES": $categories = $item; break;
	    			default: break;
	    		}
	    	}
	    	$this->setTime($id, $start, $end);
	    	$this->editEvent(
	    		$id, $summary, $description, $location, $categories);
	    }
  	}
  }

  public function removeEvent($id) {
	if (!empty($this->rcmail->user->ID)) {
	  	if (! $this->vevent)
	    	$this->vevent = $this->cal->GetEntryByUid($id);
	    $this->cal->DoDELETERequest(
	    	$this->vevent[0]['href'], $this->vevent[0]['etag']);
	}
  }
  
  public function getEvents($start, $end) {
    if (!empty($this->rcmail->user->ID)) {
      // Fetch events.
      $result = $this->cal->GetEvents($this->GMT_to_iCalendar($start), $this->GMT_to_iCalendar($end), $this->calendar);
      $events = array();

      foreach ($result as $k => $event) {
        $lines = explode("\n", $event['data']);

        $n = count($lines);
        $eventid = null;

		$flag = true;
		for ($i = 0; $i < $n; $i++) {
	  		if ($flag) {
	    		if (strpos($lines[$i], "BEGIN:VEVENT") === 0)
	      			$flag = false;

		    	continue;
	  		}

	  		if (strpos($lines[$i], "END:VEVENT") === 0)
	    	break;

		    if (empty($lines[$i]))
	    			continue;


	  		$tmp = explode(":", $lines[$i]);

	  		if (count($tmp) !== 2)
	    			continue;

	  		list($id, $value) = $tmp;
		
	  		if (!isset($id) || !isset($value))
	    			continue;
				  		
	  		if (is_null($eventid) && strpos($id, "UID") === 0)
	    		$eventid = $value;
	  		elseif (!isset($event['start']) &&
	  				strpos(strtoupper($id), "DTSTART") === 0) {
	    		$event['start'] = $this->iCalendar_to_Unix($value);
            
	    		// Check for all-day event.
	    		$event['all_day'] = (strlen($value) === 8 ? 1 : 0);
	  		}
	  		elseif (!isset($event['end']) &&
	  				strpos(strtoupper($id), "DTEND") === 0) {
	    		if ($event['all_day'])
					$event['end'] = $event['start'];
	    		else
	        		$event['end'] = $this->iCalendar_to_Unix($value);
	  		}
	  		elseif (!isset($event['summary']) &&
	  				strpos(strtoupper($id), "SUMMARY") === 0)
	    		$event['summary'] = $value;
	  		elseif (!isset($event['description']) &&
	  				strpos(strtoupper($id), "DESCRIPTION") === 0) {
	  			$event['description'] = str_replace('\n', "\n", $value);
	  		} elseif (!isset($event['location']) &&
	  				strpos(strtoupper($id), "LOCATION") === 0)
	    		$event['location'] = $value;
	  		elseif (!isset($event['categories']) &&
	  				strpos(strtoupper($id), "CATEGORIES") === 0)
	    		$event['categories'] = $value;
			}
	
			if (!isset($event['description']))
	  			$event['description'] = "";
			if (!isset($event['summary']))
	  			$event['summary'] = "";
			if (!isset($event['location']))
	  			$event['location'] = "";
			if (!isset($event['categories']))
				$event['categories'] = "";
        
        	$events[]=array( 
          		'event_id'    => $eventid,
          		'start'       => $event['start'],
          		'end'         => $event['end'],
          		'summary'     => $event['summary'],
          		'description' => $event['description'],
          		'location'    => $event['location'],
          		'categories'  => $event['categories'],
          		'all_day'	  => $event['all_day'],
        	);
      	}

      return $events;
    }
  }

  /**
   * Convert a GMT time stamp ('Y-m-d H:i:s') to the iCalendar format as defined in
   * RFC 5545, Section 3.2.19, http://tools.ietf.org/html/rfc5545#section-3.2.19.
   *
   * @param timestamp A GMT time stamp ('Y-m-d H:i:s')
   * @return An iCalendar time stamp, e.g. yyyymmddThhmmssZ
   */
  private function GMT_to_iCalendar($timestamp) {
    $unix_timestamp = $this->toGMT($timestamp);
    return date("Ymd", $unix_timestamp) . "T" . date("His", $unix_timestamp) . "Z";
  }

  /**
   * Convert a time stamp in iCalendar format as defined in
   * RFC 5545, Section 3.2.19, http://tools.ietf.org/html/rfc5545#section-3.2.19
   * to a Unix time stamp. Further conversion is done in jsonEvents.
   *
   * @param timestamp An iCalendar time stamp, e.g. yyyymmddThhmmssZ
   * @return A Unix time stamp
   */
  private function iCalendar_to_Unix($timestamp) {
     return $this->fromGMT($timestamp);
  }

  /**
   * Extract VEVENT from ICal
   * @param ICal ICal response from server
   * @return A VEVENT as an array
   */ 
  private function getEvent($ICal) {
    $START		= "BEGIN:VEVENT";
    $END		= "END:VEVENT";
    $response	= array();
  	
  	if (empty($ICal) || strlen($ICal) < 1)
  		return $response;
  		
  	$start = stristr($ICal, $START);
  	$end = stripos($ICal, $END);
  	if (strlen($start) < 1 || $end < 1)
  		return $response;
  	
  	$event = substr($ICal,
  		(strlen($ICal) - strlen($start)) + strlen($START),
  		$end - ((strlen($ICal) - strlen($start)) + strlen($START)));
  	
  	$tmp = explode("\n", $event);
  	foreach ($tmp as $item) {
  		if (empty($item))
  			continue;
  		$property = explode(":", trim($item));
  		list($prop, $value) = $property;
  		if (empty($prop))
  			continue;
  		$response[$prop] = $value;
  	}
	
  	return ($response) ? $response : array();
  }
 
  private function modifyEvent(
  		$id, $summary, $description, $location, $categories) {
  	$result = array();
  	$check = array(0, 0, 0, 0);
  	
    if (! $this->vevent)
    	$this->vevent = $this->cal->GetEntryByUid($id);
    if (empty($this->vevent) || $this->vevent == null) {
    	$this->vevent = null;
    	return $result;
    }
    
    $new_event = $this->getEvent($this->vevent[0]['data']);
    if (empty($new_event))
    	return $result;
      	
   	foreach ($new_event as $key => $item) {
   		$property = strstr($key, ";", TRUE);
   		if ($property === FALSE)
   			$property = $key;
   		switch (strtoupper($property)) {
   			case "SUMMARY":
   				$new_event["$key"] = $summary;
   				$check[0] = 1;
   				break;
   			case "DESCRIPTION":
   				$new_event["$key"] = $this->nl2ascii($description);
   				$check[1] = 1;
   				break;
   			case "LOCATION":
   				$new_event["$key"] = $location;
   				$check[2] = 1;
   				break;
   			case "CATEGORIES":
   				$new_event["$key"] = $categories;
   				$check[3] = 1;
   				break;
   			default: break;
   		}
   	}

	for ($i = 0; $i < count($check); $i++) {
		if (! $check[$i]) {
			switch ($i) {
				case 0: $new_event['SUMMARY'] = $summary; break;
				case 1: $new_event['DESCRIPTION'] =
					$this->nl2ascii($description); break;
				case 2: $new_event['LOCATION'] = $location; break;
				case 3: $new_event['CATEGORIES'] = $categories; break;
			}
		}
	}
   	
   	return $new_event;
  }
  
  /**
   * Update start and end time in VEVENT
   */
  private function setTime($id, $start, $end, $allDay = FALSE) {
    if (! $this->vevent)
    	$this->vevent = $this->cal->GetEntryByUid($id);
    if (empty($this->vevent) || $this->vevent == null) {
    	$this->vevent = null;
    	return;
    }
    
    $new_event = $this->getEvent($this->vevent[0]['data']);
    if (empty($new_event))
    	return;

	if ($allDay) {
		$values = $this->getAllDay($start, $end);
       	$start = $values->start;
		$end = $values->end;
	}
	else {
		if ($this->isDatesEqual($start, $end)) {
			// Equal start and end means the event will be deleted.
			// Adjust event to spand one hour
			$end = date("Y-m-d H:i:s", strtotime($end) + (60*60));
		}
		$start = $this->GMT_to_iCalendar($start);
		$end = $this->GMT_to_iCalendar($end);
	}
    	
   	foreach ($new_event as $key => $item) {
   		$property = strstr($key, ";", TRUE);
   		if ($property === FALSE)
   			$property = $key;
   		switch (strtoupper($property)) {
   			case "DTSTAMP":
   				$new_event["DTSTAMP"] =
   					$this->GMT_to_iCalendar(date('Y-m-d H:i:s', time()));
   				break;
   			case "DTSTART":
   				$new_event["DTSTART"] = $start;
				break;
   			case "DTEND":
   				$new_event["DTEND"] = $end;
   				break;
   			default: break;
   		}
   	}
   	$this->vevent[0]['data'] = "BEGIN:VEVENT\r\n" .
   		$this->implode_hash($new_event) . "END:VEVENT";
  }
  
  /**
   * Convert a hash array to a string
   */
  private function implode_hash(array $hash, $delim = ":", $end = "\r\n") {
  	$result = "";
  	
  	foreach ($hash as $key => $value) {
  		$key = trim($key);
  		$value = trim($value);
		$result .= "$key$delim$value$end";
	}

	return $result;
  }

  /**
   * Convert a GMT time to local
   * @param datetime A GMT time stamp ('Y-m-d H:i:s')
   * @return iCal datetime
   */
  private function GMT_to_local($datetime) {
	$lt = localtime(strtotime($datetime));
	$local = ($lt[5]+1900);
	$local .= (($lt[4]+1) < 10) ? "0".($lt[4]+1) : ($lt[4]+1);
	$local .= (($lt[3]+1) < 10) ? "0".($lt[3]+1) : ($lt[3]+1);
	
	return $local;
  }
  
  /**
   * Get start and end for an all-day event
   * @param start A GMT time stamp ('Y-m-d H:i:s')
   * @param end A GMT time stamp ('Y-m-d H:i:s')
   * @return stdClass (obj->start and obj->end)
   */
  private function getAllDay($start, $end) {
	// all day apparently delivered as UTC
	$res = new stdClass;
	
   	$res->start = $this->GMT_to_local($start);

	$tmp = date("Y-m-d H:i:s", strtotime($end) + (60*60*24));
	$res->end = $this->GMT_to_local($tmp);
	
	return $res;
  }

  /**
   * Is this an all-day event
   * @param start A GMT time stamp ('Y-m-d H:i:s')
   * @param end A GMT time stamp ('Y-m-d H:i:s')
   * @return bool true for all-day event false otherwise
   */
  private function isAllDay($start, $end) {
  	$allDay = 0;
  	
	$localStart = $this->fromGMT($start);
	$localEnd = $this->fromGMT($end);

  	// Is start and end = midnight
  	if (date('His',$localStart) != '000000' ||
  			date('His',$localEnd) != '000000')
  		return $allDay;
  	// Is diff exactly 1 day
  	$allDay = ($localEnd - $localStart) / 86400;
  	
  	return ($allDay < 1.0) ? 0 : 1;
  }

  private function fromGMT($datetime) {
    if ($this->rcmail->config->get('timezone') === "auto") {
      $tz = isset($_SESSION['timezone']) ? $_SESSION['timezone'] : date('Z')/3600;
    } else {
      $tz = $this->rcmail->config->get('timezone');
      if($this->rcmail->config->get('dst_active')) {
        $tz++;
      }
    }
    // compensate for servers timezone
    $tz -= date('Z')/3600;
    
    $timestamp = strtotime($datetime) + ($tz * 3600);
    
    return $timestamp;
  }  

  private function toGMT($datetime) {
    if ($this->rcmail->config->get('timezone') === "auto") {
      $tz = isset($_SESSION['timezone']) ? $_SESSION['timezone'] : date('Z')/3600;
    } else {
      $tz = $this->rcmail->config->get('timezone');
      if($this->rcmail->config->get('dst_active')) {
        $tz++;
      }
    }
    // compensate for servers timezone
    $tz -= date('Z')/3600;
    
    $timestamp = strtotime($datetime) - ($tz * 3600);
    
    return $timestamp;
  }
  
  private function adjustTime($start, $end) {
  	// $start and $end will hold datetime for the
  	// day to convert to all day event
  	$after = 1 + date("G", strtotime($start));
  	$s = date("Y-m-d H:i:s", $this->toGMT($start) - ($after * 60 * 60));
  	$before = 23 - date("G", strtotime($end));
  	$e = date("Y-m-d H:i:s", $this->toGMT($end) + ($before * 60 * 60) - (60*60*24));

  	return array($s, $e);
  }  

  private function isDatesEqual($date1, $date2) {
  	return strtotime($date1) === strtotime($date2);
  }
  
  private function nl2ascii($string) {
  	$ascii = "";
  	
  	$parts = explode("\n", $string);
  	for($i = 0; $i < count($parts); $i++) {
  		$part = $parts[$i];
  		if (empty($part))
  			$ascii .= '\n';
  		else if ($i + 1 < count($parts))
  			$ascii .= $part . '\n';
  		else
  			$ascii .= $part;
  	}
  	
  	return $ascii;
  }
  
}
?>
